use crate::IRect;
use glam::IVec2;

pub trait IVec2Helper {
    fn iter_to(self, other: IVec2) -> IVec2Iter;
    fn size(&self) -> usize;
    fn to_rect(self) -> IRect;
}

pub struct IVec2Iter {
    start: IVec2,
    end: IVec2,
    current: IVec2,
}

impl IVec2Helper for IVec2 {
    fn iter_to(self, other: IVec2) -> IVec2Iter {
        IVec2Iter {
            start: self,
            end: other,
            current: IVec2 {
                x: self.x - 1,
                y: self.y,
            },
        }
    }

    fn size(&self) -> usize {
        (self.x * self.y) as usize
    }

    fn to_rect(self) -> IRect {
        IRect {
            pos: IVec2::ZERO,
            size: self,
        }
    }
}

impl Iterator for IVec2Iter {
    type Item = IVec2;

    fn next(&mut self) -> Option<Self::Item> {
        self.current.x += 1;
        if self.current.x >= self.end.x {
            self.current.y += 1;
            self.current.x = self.start.x;
            if self.current.y >= self.end.y {
                return None;
            }
        }
        Some(self.current)
    }
}
